#!/bin/bash
# Script to run create image step
# Under fakeroot
# minimal-rootfs/internal
# step: following task: BUILD
# step: Previous tasks: POPULATE ROOTFS

if [ -r "$PLATFORM_LOC/scripts/utils.sh" ]; then
	source "$PLATFORM_LOC/scripts/utils.sh"
else
	source "$script_lib_dir/utils.sh"
fi

require_script rootfs_utils.sh

here=$(pwd)
basedir=$(dirname "$0")
outputDir=$here/output
rootfsDir=$outputDir/rootfs
imagesDir=$outputDir/images
bootDir=$outputDir/boot
tmpDir=$workDir/create_image.$$ # Place tmpDir in the workDir to allow cpio to use hard links instead of copying files
record_cleanup rm -rf "$tmpDir"
tmpRoot=$tmpDir/rootfs

: ${machine:=sw4l}
date=$(date +%Y%m%d%H%M%S)

: ${include_boot:=false}

# Scale factor to get valid rootfs image size
# Note: The image size has to be bigger than the folder size, it ensure no error,
# 		The user can manually reduce image size afterward
: ${ROOTFS_SCALE_FACTOR:=1.3}

# Create tmp working directory
mkdir -p $imagesDir $tmpDir

# Remove tmp working directorn on script exit
record_cleanup "rm -rf $tmpDir"

# Default parameters
: ${rootfs_image_type:=tar.bz2}

#########################################################################
# Splitted image may be a list of specifications rootDir:format:name
# 	If format is omitted, tarball is assumed
# 	If name is omitted, the basename of the directory is used (rootfs for /)
#########################################################################
analyze_fstab() {
	local fstab="$rootfsDir/etc/fstab"
	if [ ! -s $fstab ]; then
		echo "No fstab file to analyze in '$rootfsDir/etc/fstab'"
		exit 3
	fi
	
	# Reading fstab file, skipping comments
	tac "$fstab" | grep -v '^#.*$' | while read fsDev fsMnt fsType fsOtps fsDump fsPass; do
		# Only keeping non-virtual fs			
		if [[ "$fsDev" =~ ^/dev/ || "$fsDev" =~ ^UUID= || "$fsDev" =~ ^LABEL= ]]; then
			if [ "$fsType" == "swap" -o "$fsDev" = "none" ]; then continue; fi		
			echo -n "$fsMnt:$fsType:$fsDev "
		fi
	done
}

#########################################################################
# Generate image from input directory and format
# If not defined, the size is automatically set
# 1: Root directory absolute path
# 2: Image format
# 3: Output image name
#########################################################################
generate_image() {
	local imgRoot="$1"
	local imgType="${2:-tarball}"
	local imgName="$3"
	local imgSize

	# For "one image" case, no name provided
	if [ -z "$imgName" ]; then
		imgName=$(basename $imgRoot)
		imgSize=${output_size}
	fi

	# Convert into valid file name (remove / and special characters)
	if [[ "$imgName" =~ ^/ ]]; then
		imgName=$(convert_name $imgName)
	fi
	
	# Get the output image size from variable
	if [ -z "$imgSize" ]; then
		varSize=${imgName}_size
		imgSize=${!varSize}
	fi
	
	# If size equals 0, set default size which is folder size multiplied by ratio
	if [ -z "$imgSize" ] || [ $imgSize -eq 0 ]; then
		defaultSize=$(du -ks ${imgRoot} | sed 's/\t.*//')    #We know that the integer count will be follow by a \t and then some crap
		defaultSize=$(echo "${defaultSize:-4}*${ROOTFS_SCALE_FACTOR}" | bc)
		imgSize=${defaultSize%%.*}
	fi
	
	# Add machine and date to the output name
	imgFullName="${PACKAGE_ID}-${imgName}-${machine}-${date}"
	imgLinkName="${PACKAGE_ID}-${imgName}-${machine}-latest"
	
	# Generate the image
	log_exec generate_rootfs $imgRoot $imgType $imgFullName $imgName $imgSize
	
	# Find the latest generated image
	latestImagePath=$(find ${imagesDir} -name "${imgFullName}.*")
	latestImage="${latestImagePath##*/}"
	imgExtension="${latestImage#${imgFullName}.}"
	
	# Create symbolic link to latest image
	log "Create symbolic link to latest image"
	log_exec ln -nfs "${imagesDir}/${latestImage}" "${imagesDir}/${imgLinkName}.${imgExtension}"
}

#########################################################################
# File system name input human readable
#########################################################################
convert_name() {
	local fsName="$1"
	if [ "$fsName" == "/" ]; then echo "rootfs";
	else
		echo ${fsName:1} | sed -e 's/\//_/g'
	fi
}

do_buildstep_prepend

# Create a rootfs temporary working dir
#tar cf - -C $outputDir rootfs | tar xf - -C $tmpDir
verbose_errors=true
copy_tree_excluding $outputDir/rootfs $tmpDir/rootfs

# Clean output/boot folder if exists
if [ -d "$bootDir" ]; then 
	rm -rf $bootDir
fi

# Move /boot out of image if asked
# otherwise, duplicated it into output directory
if [ "$include_boot" = "false" ]; then 
	mv $tmpRoot/boot $outputDir
else
	cp $tmpRoot/boot $outputDir
fi

if [ "$rootfs_image_type" = "splitted" ]; then
	# Analyze the /etc/fstab file and generate appropriatly formatted images
	
	# Splitted image may be a list of specifications rootDir:format:name
	# 	If format is omitted, tarball is assumed
	# 	If name is omitted, the basename of the directory is used (rootfs for /)
	splitted_images=${splitted_images:-$(analyze_fstab)}
	
	if [ -z "$splitted_images" ]; then
		fatal 1 "No image splitting defined... Can't generate splitted image"
	fi
	
	while read imgRoot imgType imgName; do
		if [ "$imgName" == "/" ]; then
			rootfsName = $imgName
			rootfsType = $rootType
			rootfsRoot = $tmpRoot/$imgRoot
			continue
		fi

		
		if [ -d "$tmpRoot/$imgRoot" ]; then 
			# Generate the partition image
			generate_image $tmpRoot/$imgRoot $imgType $imgRoot
			# Remove the folder from temporary rootfs working dir
			rm -rf $tmpRoot/$imgRoot/* $tmpRoot/$imgRoot/.[^.]* $tmpRoot/$imgRoot/..?*
		else
			fatal 2 "Cannot find partition $imgName at \"$imgRoot\""
		fi
	done < <(for image_spec in $splitted_images; do
		echo $image_spec 
	done | sed -e 's/:/ /g' | sort -r -k 1 | uniq)

	if [ "$rootfsName" = "/" ]; then
		generate_image $tmpRoot $rootfsType $rootfsRoot
	fi
else
	for output_format in $rootfs_image_type; do
		generate_image $tmpRoot $output_format 
	done
fi

do_buildstep_append
